use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.006008
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib  ) );
__DATA__
ESC
Epson
baudrate
heatInterval
heatTime
heatingDots
inkjet
msg
nH
nL
param
refactor
usb
atttt
bhadoria
barcode
doublestrike
POS
att
cutpaper
dott
deviceFilePath
deviceIP
devicePort
driverType
funtions
printerIp
printerPort
CODABAR
ITF
Upto
charSpacing
charachter
cr
doesnt
drawerKickPulse
errorStatus
inkStatusA
inkStatusB
lf
lineSpacing
offlineStatus
paperSensorStatus
params
printAreaWidth
printImage
printNVImage
printPosition
printerStatus
printmode
selectDefaultLineSpacing
tabPositions
upsideDown
Sinocan
heightStatus
widthStatus
emphasizedStatus
fontStyle
underlineStatus
usePrintMode
readConstTime
productId
vendorId
serialOverUSB
driverTypes
fontHeight
fontWidth
leftMargin
customizations
optimizations
VendorId
endPoint
lsusb
portName
Shantanu
Bhadoria
shantanu
https
cpan
org
lib
Printer
ESCPOS
Connections
File
Network
Serial
USB
Manual
Profiles
Generic
SinocanPSeries
Roles
Connection
Profile
